(defun load-evaluer-elisp ()
  (interactive)
  (load "evaluer-elisp")
  (évaluer-ajout-keybinding))

;; inferior-emacs-lisp-mode-map

(defun repeat-command (command)
  "Repeat COMMAND."
  (require 'repeat)
  (let ((repeat-previous-repeated-command  command)
        (repeat-message-function           #'ignore)
        (last-repeatable-command           'repeat))
    (repeat nil)))


(defun évaluer-ajout-keybinding()
  (local-set-key (kbd "C-c C-c") 'eval-buffer)
  (local-set-key (kbd "C-x C-a") 'évaluer-sexp)
  (local-set-key (kbd "C-x a") 'évaluer-sexp-répéter)
  (local-set-key (kbd "<f5>") 'évaluer-fonction)
  (local-set-key (kbd "C-x <f5>") 'évaluer-fonction-répéter)
  (local-set-key (kbd "<f6>") 'évaluer-précédant)
  (local-set-key (kbd "C-x <f6>") 'évaluer-précédant-debug)
  (local-set-key (kbd "<f7>") 'évaluer-fonction-et-précédant)
  (local-set-key (kbd "C-x <f7>") 'évaluer-fonction-et-précédant-debug)
  (local-set-key (kbd "<f8>") 'évaluer-effacer)
  (local-set-key (kbd "C-x <f8>") 'évaluer-init)
  (local-set-key (kbd "C-h <f5>") 'shortdoc-display-group)
  )



(defvar ielm-header-perso
      "*** REPL pour Emacs LISP *** (decribe-mode) pour obtenir de l’aide"
      "Joue le même rôle que ielm-header mais on l’affiche après avoir
effacer la fenêtre")

(setq ielm-header (concat ielm-header-perso "\n"))

(setq ielm-prompt "\neλisp> ")

(defun évaluer-init ()
  (interactive)
  (delete-other-windows)
  (split-window-right)
  (other-window 1)
  (ielm)
  (split-window-below)
  (other-window 1)
  (switch-to-buffer "*Help*")
  (other-window 1))

(defun évaluer-init-effacer ()
  (interactive)
  (évaluer-init)
  (évaluer-effacer))

(defun évaluer-effacer ()
  (interactive)
  (let ((sauv (buffer-name)))
    (switch-to-buffer-other-window "*ielm*")
    (ielm)
    (goto-char (point-max))
    (insert "(progn (comint-clear-buffer) (princ ielm-header-perso))")
    (ielm-return-for-effect)
    (ring-remove comint-input-ring (ring-length comint-input-ring))
    (switch-to-buffer-other-window sauv)))


(defun affiche-old (&optional msg fin)
  "Si MSG‌ vaut nil, vide le buffer de *ielm*
Attention, ..."
  (if (null msg)
      (progn
        (unless (string= "*ielm*" (buffer-name))
          (switch-to-buffer-other-window "*ielm*")
          (ielm))

        (comint-clear-buffer))


    (let* ((fin (or fin "\n"))
           (msg (concat msg fin)))
      (princ msg))))



(defun affichef (fmt &rest valeurs)
  (let ((chaîne (apply #'format (cons fmt valeurs))))
    (affiche chaîne "")))


(defun affiche (&rest arguments)
  (let ((texte (car arguments))
        (valeurs (cdr arguments)))
        (if (null arguments)
            (progn
              (unless (string= "*ielm*" (buffer-name))
                (switch-to-buffer-other-window "*ielm*")
                (ielm))
              (comint-clear-buffer))
          (let* ((fin (if valeurs "" "\n"))
                 (texte (concat texte fin))
                 (chaîne (apply #'format (cons texte valeurs))))
            (princ chaîne))))
  nil)


(defun évaluer-fonction ()
  (interactive)
  (eval-defun nil))

(defun évaluer-fonction-répéter ()
  (interactive)
  (repeat-command 'évaluer-fonction)
  (end-of-defun))

(defun évaluer-sexp()
  (interactive)
  (let ((sauv (buffer-name))
	(beg (point)))
    (backward-sexp)
    (kill-ring-save beg (point))
    (switch-to-buffer-other-window "*ielm*")
    (ielm)
    (goto-char (point-max))
    (yank)
    (ielm-return)
    (switch-to-buffer-other-window sauv)
    (goto-char  beg)))

(defun évaluer-sexp-répéter()
  (interactive)
  (repeat-command 'évaluer-sexp)
  (forward-sexp))

(defun évaluer-précédant ()
  (interactive)
  (let ((sauv (buffer-name)))
    (switch-to-buffer-other-window "*ielm*")
    (ielm)
    (comint-previous-input 1)
    (ielm-return)
    (ring-remove comint-input-ring (ring-length comint-input-ring))
    (switch-to-buffer-other-window sauv)))

(defun évaluer-précédant-debug()
  (interactive)
  (let ((sauv (buffer-name))
	(beg nil)
        (sexpr nil))
    (save-excursion


      (switch-to-buffer-other-window "*ielm*")
      (ielm)
      (setq sexpr (read
                   (substring-no-properties
                    (ring-ref comint-input-ring 0))))
      (switch-to-buffer-other-window sauv)
      (let ((eval-expression-debug-on-error t))
        (eval-expression sexpr)))))

(defun évaluer-fonction-et-précédant()
  (interactive)
  (évaluer-fonction)
  (évaluer-précédant))

(defun évaluer-fonction-et-précédant-debug ()
  (interactive)
  (évaluer-fonction)
  (évaluer-précédant-debug))



;; (defmacro common-lisp (sexpr)
;;   (let ((chaîne (format "%S" sexpr)))
;;     `(slime-eval (quote (cl:eval (cl:read-from-string ,chaîne))) "CL-USER")))


(defmacro common-lisp (sexpr &optional étapes)
  (let ((chaîne (format "%S" sexpr))
        (étapes (or étapes 6)))
    `(let ((attente 0.0005)
           (n 0)
           (étapes ,étapes)
           (résultat 'aucun-common-lisp))

       (sly-eval-async (quote (cl:eval (cl:read-from-string ,chaîne)))
         (lambda (x) (setq résultat x))
         "VALROSE")

     (sleep-for attente)
     (catch 'erreur
       (while (eq 'aucun-common-lisp résultat)
         (setq attente (* 2 attente))
         (when (<= étapes n)
           (message (format "Attente du résultat (%i/%i) ERREUR" ,étapes ,étapes))
           (throw 'erreur 'erreur-common-lisp))
         (message (format "Attente du résultat (%i/%i)" n ,étapes))
         (setq n (+ 1 n))
         (sleep-for attente))
       (message (format "OK %i/%i" n , étapes))
       résultat))))


(defun common-lisp-args (fonction args &optional étapes)
  (let ((chaîne (format "%S" (cons fonction args)))
        (étapes (or étapes 6)))
    (let ((attente 0.0005)
          (n 0)
          (résultat 'aucun-common-lisp))

      (sly-eval-async `(cl:eval (cl:read-from-string ,chaîne))
        (lambda (x) (setq résultat x))
        "VALROSE")

      (sleep-for attente)
      (catch 'erreur
        (while (eq 'aucun-common-lisp résultat)
          (setq attente (* 2 attente))
          (when (<= étapes n)
            (message (format "Attente du résultat (%i/%i) ERREUR" étapes étapes))
            (throw 'erreur 'erreur-common-lisp))
          (message (format "Attente du résultat (%i/%i)" n étapes))
          (setq n (+ 1 n))
          (sleep-for attente))
        (message (format "OK %i/%i" n  étapes))
       résultat))))


(defun pilou-script ()
  (interactive)
  (lisp-mode)
  (goto-char 0)
  (replace-string ";; -*- lexical-binding: t; -*-"
                  (concat "(load \"/home/olivier/dév/pilou/pilou.lisp\")"
                          "\n"
                          "(in-package :pilou)")))



(provide 'évaluer-elisp)
