(defmacro brouillon (&rest _)
  nil)

(defun load-evaluer-python ()
  (interactive)
  (load "evaluer-python")
  (évaluer-ajout-keybinding))


(defun évaluer-ajout-keybinding()
  (local-set-key (kbd "C-x C-a") 'valrose-python-send)
  (local-set-key (kbd "C-x a")   'valrose-python-send-and-repeat)
  (local-set-key (kbd "<f5>")    'valrose-python-send-defun)
  (local-set-key (kbd "<f9>")    'valrose-python-clear-buffer)
  (local-set-key (kbd "C-x <f9>")    'valrose-python-restart-buffer)
  (local-set-key (kbd "<f6>")    'valrose-python-send-last)
  (local-set-key (kbd "<f7>")    'valrose-python-send-defun-and-last))


(defvar *calcul-en-cours* nil)
(defvar *dernière-commande* "None")
(defvar *dernière-commande-ligne* t) ;; nil si région


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                          ;;
;;  Fonctions auxiliaires   ;;
;;                          ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun valrose-python-add-new-line (texte)
  (when (< 4 (length texte)) ; ">>> "
    (with-current-buffer "*Python*"
      (goto-char (point-max))
      (sleep-for 0.0)
      (forward-line -1)
      (end-of-line)
      (insert "\n")
      (goto-char (point-max)))))

(defun valrose-python-terminer-calcul (texte)
  (setq *calcul-en-cours* nil))


;; (comint-check-proc "*Python*")
(defun valrose-python-run-if-needed ()
  (interactive)
  (unless (and (get-buffer "*Python*")
               (comint-check-proc "*Python*"))
    (let ((sauv (buffer-name))
          (courant (point)))
      (split-window-right)
      (run-python)
      (switch-to-buffer-other-window "*Python*")
      (add-hook 'comint-output-filter-functions 'valrose-python-add-new-line 0 t)
      (add-hook 'comint-output-filter-functions 'valrose-python-terminer-calcul 0 t)
      (switch-to-buffer-other-window sauv)
      (goto-char courant))

    (sleep-for .1)))


;;      (add-hook 'comint-output-filter-functions #'valrose-python-add-new-line)
(defun valrose-python-get-function-name ()
  (save-excursion
    (while (and (< 1 (point))
                (or (commence-par " " (thing-at-point 'line))
                    (valrose-python-empty-line-p)))
      (forward-line -1))

    (let* ((ligne-def (thing-at-point 'line))
           (bon-début (commence-par "def " ligne-def))
           (parenthèse ?\()
           (indice (seq-position ligne-def parenthèse)))
      (if (and bon-début indice)
          (substring-no-properties ligne-def 4 indice)
        nil))))

(defun valrose-python-empty-line-p ()
  (let ((ligne (thing-at-point 'line)))
    (or (commence-par "#" ligne)
        (cl-loop for c across ligne
                 always (or (eq c ?\ )
                            (eq c ?\n)
                            (eq c ?\t))))))

(defun valrose-python-ligne-hors-bloc-p ()
  (let ((ligne (thing-at-point 'line)))
    (or (not (commence-par " " ligne))
        (commence-par "else " ligne))))

(defun valrose-python-ligne-début-bloc-p ()
  (let ((ligne (thing-at-point 'line)))
    (and (not (valrose-python-empty-line-p))
         (not (commence-par " " ligne))
         (not (commence-par "else" ligne)))))




(defun valrose-python-get-block ()
  (let (début fin curseur)
    ;; trouver début
    (save-excursion
      (while (and (< 1 (point))
                  (not (valrose-python-ligne-début-bloc-p)))
        (forward-line -1))

      (when (valrose-python-ligne-début-bloc-p)
        (setq début (progn (beginning-of-line) (point)))))


    (save-excursion
      (when (valrose-python-ligne-début-bloc-p)
        (forward-line 1))
      (setq curseur (point))
      (while (and (< (point) (point-max) )
                  (not (valrose-python-ligne-début-bloc-p)))
        (when (not (valrose-python-empty-line-p))
          (setq curseur (point)))
        (forward-line 1))

      (when (valrose-python-ligne-début-bloc-p)
        (forward-line -1))
      (setq fin (progn  (goto-char curseur) (end-of-line) (point))))


    (if (and début fin)
        (cons début fin)
      (message "Erreur bloc %s %s" début fin))))




(defun valrose-python-send-block-or-defun (&optional goto-next)
  (let* ((bornes (valrose-python-get-block))
         (début (car bornes))
         (fin   (cdr bornes)))
    (if (and début fin)
        (progn
          (save-excursion
            (goto-char début)
            (if (commence-par "def " (thing-at-point 'line))
                (valrose-python-send-defun début fin)
              (valrose-python-send-region début fin))))
      (message "Erreur %s %s" début fin))

    (when (and début fin goto-next)
      (goto-char fin)
      (forward-line 1)
      (while (and (< (point) (point-max) )
                  (valrose-python-empty-line-p))
        (forward-line 1)))))



(defun string-supprimer-ligne-vide (string)
  (replace-regexp-in-string "^[ ]*\n" "" string))



(defun valrose-python-send-defun (début fin)
  (interactive)
    (let ((sauv (buffer-name))
          (courant (point)))

      (valrose-python-run-if-needed)


      (let ((nom-fonction (valrose-python-get-function-name)))
        (if nom-fonction
            (progn
              (python-shell-send-string-no-output
               (string-supprimer-ligne-vide
                (buffer-substring-no-properties début fin)))
              ;(python-shell-send-defun)
              (sleep-for .1)
              (with-current-buffer "*Python*"
                (goto-char (point-max))
                (comint-send-input)
                (forward-line -1)
                (let ((inhibit-field-text-motion t))
                  (end-of-line))
                (let ((inhibit-read-only t))
                  (insert
                   (propertize
                    (format "Définition de la fonction « %s »\n"
                            nom-fonction)
                    'font-lock-face 'comint-highlight-prompt))))

              (with-selected-window (get-buffer-window "*Python*")
                (with-current-buffer "*Python*"
                  (recenter (window-body-height) t))))
          (message "Pas une fonction"))
        (redisplay t)
      (goto-char courant))))


;; Poubelle ?
(defun valrose-python-send-block ()
  (interactive)
    (let ((sauv (buffer-name))
          (courant (point)))

      (valrose-python-run-if-needed)


      (let ((nom-fonction (valrose-python-get-function-name)))
        (if nom-fonction
            (progn
              (python-shell-send-defun)
              (sleep-for 0.1)
              (with-current-buffer "*Python*"
                (goto-char (point-max))
                (forward-line -1)
                (let ((inhibit-field-text-motion t))
                  (end-of-line))
                (let ((inhibit-read-only t))
                  (message "Avant")
                  (insert
                   (propertize
                    (format "Définition de la fonction « %s »\n"
                            nom-fonction)
                    'font-lock-face 'comint-highlight-prompt))
                  (message "après"))))
          (message "Pas une fonction"))
        (redisplay t)
      (goto-char courant))))



(defun valrose-python-send-line (&optional goto-next)
  (interactive)
  (let* ((début (line-beginning-position))
         (fin   (line-end-position))
         (ligne (conserver-propriétés
                 (buffer-substring début fin)))) ;; STRNG-TRIM
    (setq *dernière-commande-ligne* t)
    (setq *dernière-commande* ligne)

    (let ((sauv (buffer-name))
          (courant (point)))
      (valrose-python-run-if-needed)
      (switch-to-buffer-other-window "*Python*")
      (goto-char (point-max))
      (insert ligne)
      (redisplay t)
      (comint-send-input)
      (switch-to-buffer-other-window sauv)
      (goto-char courant)))

  (when goto-next
    (forward-line)
    (while (and (< (line-end-position) (point-max))
                (= (line-beginning-position) (line-end-position)))
      (forward-line))))


(defun commence-par (chaîne texte)
  (let ((k (length chaîne))
        (n (length texte)))
    (if (< n k)
        nil
      (string= chaîne (substring texte 0 k)))))

(defun fini-par (chaîne texte)
  (let ((k (length chaîne))
        (n (length texte)))
    (if (< n k)
        nil
      (string= chaîne (substring texte (- n k) n)))))





(defun valrose-python-send()
  (interactive)
  (if (region-active-p)
      (valrose-python-send-region)
    (let ((ligne (thing-at-point 'line)))
      (if (or (fini-par ":\n" ligne)
              (commence-par " " ligne)
              (commence-par "\t" ligne))
          (valrose-python-send-block-or-defun)
        (valrose-python-send-line)))))

(defun valrose-python-send-and-move()
  (interactive)
  (if (region-active-p) ;; dans ce cas on ne bouge pas
      (valrose-python-send-region)
    (let ((ligne (thing-at-point 'line)))
      (if (or (fini-par ":\n" ligne)
              (commence-par " " ligne)
              (commence-par "\t" ligne))
          (valrose-python-send-block-or-defun t)
        (valrose-python-send-line t)))))


(defun valrose-python-send-last ()
  (interactive)
  (valrose-python-send-string *dernière-commande*))

;; poubelle
(defun valrose-python-send-line-répéter ()
  (interactive)
  (repeat-command 'valrose-python-send-line)
  (forward-line)
  (while (and (< (line-end-position) (point-max))
              (= (line-beginning-position) (line-end-position)))
    (forward-line)))

(defun valrose-python-send-and-repeat ()
  (interactive)
  (repeat-command 'valrose-python-send-and-move))



(defun valrose-python-clear-buffer ()
  (interactive)
  (when (get-buffer "*Python*")
    (let ((sauv (buffer-name))
          (courant (point)))
      (switch-to-buffer-other-window "*Python*")
      (comint-clear-buffer)
      (switch-to-buffer-other-window sauv)
      (goto-char courant))))


(defun valrose-python-restart-buffer ()
  (interactive)
  (let ((sauv (buffer-name))
        (courant (point)))
    (when (get-buffer "*Python*")
      (with-current-buffer "*Python*"
        (comint-quit-subjob)))
    (let ((kill-buffer-query-functions nil))
      (kill-buffer "*Python*"))
    (run-python)
    (delete-other-windows)
    (switch-to-buffer-other-window "*Python*")
    (switch-to-buffer-other-window sauv)
    (goto-char courant)))



(defun valrose-python-send-region (&optional début fin)
  (interactive)
  (valrose-python-send-string (buffer-substring (or début (region-beginning))
                                                (or fin (region-end)))))



(defun valrose-python-send-string (code)
  (let ((sauv (buffer-name))
        (courant (point)))

    (valrose-python-run-if-needed)

    (setq *dernière-commande-ligne* nil)
    (setq *dernière-commande* code)

    (switch-to-buffer-other-window "*Python*")
    (goto-char (point-max))
    (setq code (split-string-with-properties code))
    (dolist (ligne code)
      (insert (conserver-propriétés ligne))
      (sleep-for 0.01)
      (comint-send-input))
    (when (cdr code)
      (sleep-for 0.01)
      (comint-send-input))
    (switch-to-buffer-other-window sauv)
    (goto-char courant)))

    ;;     (setq *dernière-commande-ligne* nil)
    ;;     (setq *dernière-commande* code)

    ;;   (switch-to-buffer-other-window "*Python*")
    ;;   (goto-char (point-max))
    ;;   (setq code (split-string-with-properties code))
    ;;   (dolist (ligne code)
    ;;     (insert (conserver-propriétés ligne))
    ;;     (sleep-for 0.01)
    ;;     (comint-send-input))
    ;;   (when (cdr code)
    ;;     (sleep-for 0.01)
    ;;     (comint-send-input)))
    ;; (switch-to-buffer-other-window sauv)
    ;; (goto-char courant)))

(defun split-string-with-properties (chaîne &optional sep)
  (let ((sep (or sep ?\n)))
    (seq-filter (lambda (s) (< 0 (length s)))
                (cl-loop with début = 0
                         for c across chaîne
                         for i from 0
                         when (= ?\n c)
                         collect (substring chaîne  début i) into rés
                         and do (setq début (+ 1 i))
                         finally
                         (return (nconc rés (list (substring chaîne  début i))))))))

(brouillon
 (split-string-with-properties "1+1")
 (split-string-with-properties "\naze\n123\nsalut\n")
 (split-string "\naze\n123\nsalut\n")
 nil)


(defun remplacer (liste ancien nouveau)
  (cl-loop for x in liste
           if (eq x ancien)
           collect nouveau
           else
           collect x))

(brouillon
 (remplacer '(toto titi tata titi) 'titi 'youpi)
 nil)

(defun conserver-propriétés(ligne)
  (interactive)
  (if (object-intervals ligne)
      (let ((prop   (cl-loop for (a b c)  in (object-intervals ligne)
                             for cc = (remplacer c 'face 'font-lock-face)
                             collect (list a b cc))))

        (apply #'concat
               (cl-loop for (a b c) in prop
                        collect
                        (apply #'propertize (cons (substring-no-properties ligne a b)
                                                  c)))))
    ligne))

(provide 'évaluer-python)
