
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                        ;;
;;  Fonctions générales   ;;
;;                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun upinfo/white-space ()
  (interactive)
  (setq-default whitespace-style
		;; value empty : remove empty line at beginnig or end of buffer
		'(face tabs spaces trailing lines space-before-tab newline
		       empty space-after-tab space-mark tab-mark newline-mark) )
  (add-hook 'before-save-hook 'whitespace-cleanup)
  (global-set-key (kbd "C-C w") 'whitespace-mode))

(defun upinfo/line-number ()
  (interactive)
  (display-line-numbers-mode))

(defun upinfo/no-tab-for-indent ()
  (setq-default indent-tabs-mode nil))

(defun upinfo/comment ()
  (interactive)
  (global-set-key (kbd "C-C ;") 'comment-or-uncomment-region))

(defun upinfo/hs ()
  (interactive)
  (hs-minor-mode)
  (local-set-key (kbd "C-C RET") 'hs-toggle-hiding)
  (setq hs-hide-comments-when-hiding-all nil))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                        ;;
;;  Fonctions pour le C   ;;
;;                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun upinfo/debug-c (args)
  (interactive "sVariables : ")
  (let ((args (split-string args))
        (debut "printf(\""))

    (end-of-visual-line)
    (newline)

    (dolist (first args)
      (insert debut first "=%d")
      (setq debut " "))

    (insert "\\n\"")

    (setq debut ", ")
    (dolist (first args)
      (insert debut first))

    (insert "); /* DEBUG VALROSE */")
    (c-indent-line-or-region)))


(defun upinfo/header-source  (nom-buffer)
  (let* ((n (length nom-buffer))
         (nom   (seq-subseq nom-buffer 0 (- n 2)))
         (final (seq-subseq nom-buffer (- n 2) n)))
    (cond ((string= final ".c")
           (concat nom ".h"))
          ((string= final ".h")
           (concat nom ".c"))
          (t nom-buffer))))


(defun upinfo/switch-to-header ()
  (interactive)
  (find-file (upinfo/header-source (buffer-name))))




(defun upinfo/load-debug (function-name)
  (interactive)
  (local-set-key (kbd "C-C d") function-name))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                             ;;
;;  Hook pour chaque langage   ;;
;;                             ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun upinfo/mode-python()
  (interactive)
  (setq python-shell-interpreter "python3")
  (setq python-shell-interpreter-args "-i")
  (setq confirm-kill-processes nil)
  (upinfo/hs)
  (upinfo/comment)
  (upinfo/white-space)
  (upinfo/no-tab-for-indent))

(autoload 'load-evaluer-python "evaluer-python.el" "Mode perso pour python" t)
(add-hook 'python-mode-hook 'upinfo/mode-python)
(add-hook 'python-mode-hook 'load-evaluer-python)

(defun upinfo/mode-c ()
  (interactive)
  (setq c-basic-offset 4)
  (setq c-default-style "linux")
  (upinfo/hs)
  (upinfo/comment)
  (upinfo/line-number)
  (upinfo/white-space)
  (local-set-key (kbd "C-C C-C") 'compile)
  (local-set-key (kbd "<f9>") 'upinfo/switch-to-header)
  (local-set-key (kbd "C-C d") 'upinfo/debug-c)
  (upinfo/no-tab-for-indent))

(add-hook 'c-mode-common-hook 'upinfo/mode-c)


(defun upinfo/mode-lisp ()
  (interactive)
  (upinfo/hs)
  (upinfo/comment)
  (upinfo/white-space)
  (upinfo/no-tab-for-indent))

(autoload 'load-evaluer-elisp "evaluer-elisp.el" "Mode perso pour EMACS LISP" t)
(add-hook 'emacs-lisp-mode-hook 'upinfo/mode-lisp)
(add-hook 'emacs-lisp-mode-hook 'load-evaluer-elisp)
