;; -*- lexical-binding: t; -*-


(defun baldellon-s* (chaîne n)
  (if (= 0 n)
      ""
    (concat chaîne (baldellon-s* chaîne (- n 1)))))

(defun baldellon-correspondance (décalage regexp)
  (save-excursion
    (if (/= 0 (forward-line décalage))
        nil
      (eq 0
          (string-match regexp (string-trim (thing-at-point 'line t)))))))

(defun baldellon-est-titre-ascii? (début milieu fin)
  (let ((début (regexp-quote début))
        (fin   (regexp-quote fin))
        (milieu (regexp-quote
                 (if (= (elt milieu 0) (elt milieu 1))
                     (char-to-string (elt milieu 0))
                   milieu))))
    (and
     (baldellon-correspondance -2 (concat "^" début "\\(" milieu "\\)+" fin "$"))
     (baldellon-correspondance  2 (concat "^" début "\\(" milieu "\\)+" fin "$"))
     (baldellon-correspondance -1 (concat "^" début "\\("  " "   "\\)+" fin "$"))
     (baldellon-correspondance  1 (concat "^" début "\\("  " "   "\\)+" fin "$"))
     (baldellon-correspondance  0 (concat "^" début )))))


(defun baldellon-est-commentaire? (début milieu fin)
  (let ((début (regexp-quote début))
        (fin   (regexp-quote fin))
        (milieu (regexp-quote
                 (if (= (elt milieu 0) (elt milieu 1))
                     (char-to-string (elt milieu 0))
                   milieu))))
    (and
     (baldellon-correspondance  0 (concat "^" début )))))






(defun baldellon-titre-ascii-art-supprimer (début milieu fin)
  (forward-line 3)
  (insert "\n")
  (forward-line -4)
  (let* ((début (regexp-quote début))
         (fin   (regexp-quote fin))
         (gauche (concat "\\(" début "\\|" "[ \t\n\r]" "\\)" "+" ))
         (droite (concat "\\(" fin   "\\|" "[ \t\n\r]" "\\)" "+" ))
         (ligne (thing-at-point 'line t))
         (ligne (string-trim ligne gauche droite)))
    (forward-line -2)
    (beginning-of-visual-line)
    (kill-visual-line 5)
    (insert ligne)))



(defun titre-ascii-art-général (début &optional milieu fin)
    (if (/= 2 (length début))
      (error "Le symbole doit être de longueur 1"))

    (let ((milieu (or milieu début))
          (fin    (or fin début)))
      (when (baldellon-est-titre-ascii? début milieu fin)
          (baldellon-titre-ascii-art-supprimer début milieu fin))

      (if (baldellon-est-commentaire? début milieu fin)
          (message "Pas de titre dans un commentaire")
        (let* ((espaces "  ")
               (ligne (string-trim (thing-at-point 'line t)))
               (parité (% (length ligne) 2))
               (ligne (concat ligne (baldellon-s* " " parité)))
               (longueur (+ (/ (length ligne) 2) (length espaces))))

          (beginning-of-visual-line)
          (kill-visual-line)

          (insert (concat
                   début (baldellon-s* milieu longueur)  fin "\n"
                   début (baldellon-s* espaces longueur) fin "\n"
                   début           espaces ligne espaces fin "\n"
                   début (baldellon-s* espaces longueur) fin "\n"
                   début (baldellon-s* milieu longueur)  fin ))
          (forward-line -2)
          (right-char 4))
        )))

(defun titre-ascii-art-lisp ()
  (interactive)
  (titre-ascii-art-général ";;"))

(defun titre-ascii-art-python ()
  (interactive)
  (titre-ascii-art-général "##"))

(defun titre-ascii-art-shell ()
  (interactive)
  (titre-ascii-art-général "##"))


(defun titre-ascii-art-tex ()
  (interactive)
  (titre-ascii-art-général "%%"))

(defun titre-ascii-art-c ()
  (interactive)
  (titre-ascii-art-général "/*" "**" "*/"))

(defun espace()
  "espace"
  (interactive)
  (beginning-of-line)
  (newline)
  (previous-line)
  (delete-blank-lines)
  (newline)
  (newline)
  (newline)
  (newline))

;(defalias 'titre-ascii-art 'titre-ascii-art-python)

(global-set-key (kbd "C-x p RET" ) 'espace)
;(global-set-key (kbd "C-x p TAB")  'titre-ascii-art)


(add-hook  'emacs-lisp-mode-hook
           (lambda ()
              (local-set-key (kbd "C-x p TAB") 'titre-ascii-art-lisp)))


(add-hook  'slime-mode-hook
           (lambda ()
              (local-set-key (kbd "C-x p TAB") 'titre-ascii-art-lisp)))

(add-hook  'sly-mode-hook
           (lambda ()
              (local-set-key (kbd "C-x p TAB") 'titre-ascii-art-lisp)))


(add-hook  'geiser-mode-hook
           (lambda ()
              (local-set-key (kbd "C-x p TAB") 'titre-ascii-art-lisp)))


(add-hook  'LaTeX-mode-hook
           (lambda ()
              (local-set-key (kbd "C-x p TAB") 'titre-ascii-art-tex)))


(add-hook  'c-mode-common-hook
           (lambda ()
              (local-set-key (kbd "C-x p TAB") 'titre-ascii-art-c)))


(add-hook  'python-mode-hook
           (lambda ()
              (local-set-key (kbd "C-x p TAB") 'titre-ascii-art-python)))


(add-hook  'sh-mode-hook
           (lambda ()
              (local-set-key (kbd "C-x p TAB") 'titre-ascii-art-shell)))
